library ieee;
use ieee.std_logic_1164.all;
use ieee.std_logic_arith.all;
use ieee.std_logic_unsigned.all;
use work.math_pack.all;

entity switch is
    generic (
        FREQ_CLK         : positive;          -- frekvence hodin
        LENGTH           : positive           -- delka stisku v ms
    );
    port (
        CLK              : in std_logic; 
        RST              : in std_logic;   
        SW               : in std_logic;
        PRESSED          : out std_logic;
        BUSY             : out std_logic
    ); 
end switch;



architecture switch_behavioral of switch is
    component counter is
    generic( 
        DATA_WIDTH        : integer
    );
    port( 
        CLK      : in std_logic;
        RST      : in std_logic;
        CLR      : in std_logic;
        EN       : in std_logic;
        DATA     : out std_logic_vector(DATA_WIDTH-1 downto 0)
    ); 
    end component;

    type states is (
        INIT,
        IDLE,
        SW_PRESSED
    );

    constant CYCLE_COUNT      : integer := FREQ_CLK / 1000 * LENGTH;

    signal state : states := INIT;
    signal nextstate : states := INIT;

    signal clk_sig : std_logic;        
    signal rst_sig : std_logic;

    signal wait_counter_data_sig : std_logic_vector(log2(CYCLE_COUNT) downto 0);
    signal wait_counter_en_sig, wait_counter_clr_sig : std_logic;

    signal wait_finished_sig : std_logic;
begin

    wait_counter_inst : counter
    generic map ( 
        DATA_WIDTH => log2(CYCLE_COUNT)+1
    )
    port map ( 
        CLK => clk_sig,
        RST => rst_sig,
        CLR => wait_counter_clr_sig,
        EN => wait_counter_en_sig,
        DATA => wait_counter_data_sig
    ); 
    wait_finished_sig <= '0' when (wait_counter_data_sig < CYCLE_COUNT) else '1'; 

 
    state_current : process(clk_sig,rst_sig)
    begin
        if (rst_sig='1') then
            state <= INIT;
        elsif rising_edge(clk_sig) then
            state <= nextstate;
        end if;
    end process;
     
    state_machine : process(state, SW, wait_finished_sig)
    begin
        wait_counter_clr_sig <= '0';
        wait_counter_en_sig <= '0';
        PRESSED <= '0';
        BUSY <= '0';
        nextstate <= INIT;

        case state is
		    
        when INIT =>    
            wait_counter_clr_sig <= '1';
            nextstate <= IDLE;

        when IDLE =>
            if(SW = '1') then
                wait_counter_en_sig <= '1';
                BUSY <= '1';
                nextstate <= SW_PRESSED;
            else
                nextstate <= IDLE;
            end if;  
      
        when SW_PRESSED =>
            BUSY <= '1';
            if(wait_finished_sig = '1') and (SW = '1') then
                PRESSED <= '1';
                wait_counter_clr_sig <= '1';
                nextstate <= IDLE;
            elsif(wait_finished_sig = '1') and (SW = '0') then
                wait_counter_clr_sig <= '1';
                nextstate <= IDLE;
            else
                wait_counter_en_sig <= '1';
                nextstate <= SW_PRESSED;
            end if;             

        when others => nextstate <= INIT;
				
        end case;
    end process;


    rst_sig <= RST;
    clk_sig <= CLK;

end switch_behavioral;
